function HA = steering_controller_HA()
dim = 3;

% resets are all identity
reset.A = eye(dim);
reset.b = zeros(dim,1);

% options
dynOpt = struct('tensorOrder',1);
dynOpt.uTrans = 0;
dynOpt.U = zonotope([0]);

%% ############## Location keep_lane

% flow:
dynamics = nonlinearSys(dim,1,@dynamics_keep,dynOpt); 

% invariant:
invA = [-1,0,0;...
        1,0,0];
invb = [1;...
        1];
invOpt = struct('A', invA, 'b', invb);
inv = mptPolytope(invOpt);


% keep -> left_close
guard1 = mptPolytope(struct('A', [-1 0 0],...
    'b', [-1]));
trans{1} = transition(guard1, reset, 2, 'dummy','test');

% keep -> right_close
guard2 = mptPolytope(struct('A', [1 0 0],...
    'b', [-1]));
trans{2} = transition(guard2, reset, 3, 'dummy','test');

loc{1} = location('keep_lane',1, inv, trans, dynamics);

%% ############## Location left_close

% flow:
dynamics = nonlinearSys(dim,1,@dynamics_close,dynOpt);

% invariant:
invA = [-1,0,0;...
        1,0,0];
invb = [-0.7;...
        2];
invOpt = struct('A', invA, 'b', invb);
inv = mptPolytope(invOpt);

% left_close -> keep
guard1 = mptPolytope(struct('A', [1 0 0],...
    'b', [0.7]));
trans{1} = transition(guard1, reset, 1, 'dummy','test');

% left_close -> left_mid
guard2 = mptPolytope(struct('A', [-1 0 0],...
    'b', [-2]));
trans{2} = transition(guard2, reset, 4, 'dummy','test');

loc{2} = location('left_close',2, inv, trans, dynamics);

%% ############## Location right_close

% flow:
dynamics = nonlinearSys(dim,1,@dynamics_close,dynOpt);

% invariant:
invA = [-1,0,0;...
        1,0,0];
invb = [2;...
        -0.7];
invOpt = struct('A', invA, 'b', invb);
inv = mptPolytope(invOpt);

% right_close -> keep
guard1 = mptPolytope(struct('A', [-1 0 0],...
    'b', [0.7]));
trans{1} = transition(guard1, reset, 1, 'dummy','test');

% right_close -> right_mid
guard2 = mptPolytope(struct('A', [1 0 0],...
    'b', [-2]));
trans{2} = transition(guard2, reset, 5, 'dummy','test');

loc{3} = location('right_close',3, inv, trans, dynamics);

%% ############## Location left_mid

% flow:
dynamics = nonlinearSys(dim,1,@dynamics_mid,dynOpt);

% invariant:
invA = [-1,0,0;...
        1,0,0];
invb = [-1.5;...
        7];
invOpt = struct('A', invA, 'b', invb);
inv = mptPolytope(invOpt);

% left_mid -> left_close
guard1 = mptPolytope(struct('A', [1 0 0],...
    'b', [1.5]));
trans{1} = transition(guard1, reset, 2, 'dummy','test');

% left_mid -> left_far
guard2 = mptPolytope(struct('A', [-1 0 0],...
    'b', [-7]));
trans{2} = transition(guard2, reset, 6, 'dummy','test');

loc{4} = location('left_mid',4, inv, trans, dynamics);

%% ############## Location right_mid

% flow:
dynamics = nonlinearSys(dim,1,@dynamics_mid,dynOpt);

% invariant:
invA = [-1,0,0;...
        1,0,0];
invb = [7;...
        -1.5];
invOpt = struct('A', invA, 'b', invb);
inv = mptPolytope(invOpt);

% right_mid -> right_close
guard1 = mptPolytope(struct('A', [-1 0 0],...
    'b', [1.5]));
trans{1} = transition(guard1, reset, 3, 'dummy','test');

% right_mid -> right_far
guard2 = mptPolytope(struct('A', [1 0 0],...
    'b', [-7]));
trans{2} = transition(guard2, reset, 7, 'dummy','test');

loc{5} = location('right_mid',5, inv, trans, dynamics);

%% ############## Location left_far

% flow:
dynamics = nonlinearSys(dim,1,@dynamics_far,dynOpt);

% invariant:
invA = [-1,0,0;...
        1,0,0];
invb = [-6;...
        10];
invOpt = struct('A', invA, 'b', invb);
inv = mptPolytope(invOpt);

% left_far -> left_mid
guard1 = mptPolytope(struct('A', [1 0 0],...
    'b', [6]));
trans{1} = transition(guard1, reset, 4, 'dummy','test');

loc{6} = location('left_far',6, inv, trans, dynamics);

%% ############## Location right_far

% flow:
dynamics = nonlinearSys(dim,1,@dynamics_far,dynOpt);

% invariant:
invA = [-1,0,0;...
        1,0,0];
invb = [10;...
        -6];
invOpt = struct('A', invA, 'b', invb);
inv = mptPolytope(invOpt);

% right_mid -> right_close
guard1 = mptPolytope(struct('A', [-1 0 0],...
    'b', [6]));
trans{1} = transition(guard1, reset, 5, 'dummy','test');

loc{7} = location('right_far',7, inv, trans, dynamics);


%% Assemble automaton

HA = hybridAutomaton(loc);